"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const electron_1 = require("electron");
const mkdirp_1 = require("mkdirp");
const menu_1 = require("./menu");
const browser_config_1 = require("./browser-config");
if (process.argv.indexOf('--help') !== -1) {
    console.log(`OVERVIEW: NyaoVim; Web-enhanced Extensible Neovim Frontend

USAGE: nyaovim [options] [neovim args...]

OPTIONS:
  --no-detach : Don't detach the editor process
  --help      : Show this help
  --version   : Show versions of NyaoVim, Electron, Chrome, Node.js, and V8
`);
    electron_1.app.quit();
}
if (process.argv.indexOf('--version') !== -1) {
    const vs = process.versions;
    const versions = ['electron', 'chrome', 'node', 'v8'].map((n) => `  ${n} : ${vs[n]}`).join('\n');
    console.log(`${electron_1.app.getName()} version ${electron_1.app.getVersion()}
${versions}
`);
    electron_1.app.quit();
}
process.on('unhandledRejection', (reason, p) => {
    console.error('Fatal: Unhandled rejection at: Promise', p, 'Reason:', reason);
});
const is_run_from_npm_package_on_darwin = electron_1.app.getAppPath().indexOf('/NyaoVim.app/') === -1;
const config_dir_name = process.platform !== 'darwin' ?
    electron_1.app.getPath('appData') :
    process.env.XDG_CONFIG_HOME || path_1.join(process.env.HOME, '.config');
global.config_dir_path = path_1.join(config_dir_name, 'nyaovim');
global.nyaovimrc_path = path_1.join(global.config_dir_path, 'nyaovimrc.html');
function exists(path) {
    return new Promise(resolve => {
        fs_1.stat(path, (err, stats) => {
            if (err) {
                resolve(false);
                return;
            }
            resolve(stats.isFile() || stats.isDirectory());
        });
    });
}
function prepareDefaultNyaovimrc() {
    console.log('Generate default nyaovimrc at ' + global.nyaovimrc_path);
    return exists(global.config_dir_path).then(e => {
        if (!e) {
            mkdirp_1.sync(global.config_dir_path);
        }
    }).then(() => {
        const contents = `<dom-module id="nyaovim-app">
  <template>
    <style>
      /* CSS configurations here */
    </style>

    <!-- Component tags here -->
    <neovim-editor id="nyaovim-editor" argv="[[argv]]" font="monospace"></neovim-editor>
  </template>
</dom-module>
`;
        fs_1.writeFileSync(global.nyaovimrc_path, contents, 'utf8');
    });
}
const ensure_nyaovimrc = exists(global.nyaovimrc_path).then((e) => {
    if (!e) {
        return prepareDefaultNyaovimrc();
    }
    else {
        return undefined;
    }
}).catch(err => console.error(err));
const browser_config = new browser_config_1.default();
const prepare_browser_config = browser_config.loadFrom(global.config_dir_path)
    .catch(err => console.error(err));
function startMainWindow() {
    const index_html = 'file://' + path_1.join(__dirname, '..', 'renderer', 'main.html');
    const default_config = {
        width: 800,
        height: 600,
        useContentSize: true,
        webPreferences: {
            blinkFeatures: 'KeyboardEventKey,Accelerated2dCanvas,Canvas2dFixedRenderingMode',
        },
        icon: electron_1.nativeImage.createFromPath(path_1.join(__dirname, '..', 'resources', 'icon', 'nyaovim-logo.png')),
    };
    const user_config = browser_config.applyToOptions(default_config);
    let win = new electron_1.BrowserWindow(user_config);
    const already_exists = browser_config.configSingletonWindow(win);
    if (already_exists) {
        electron_1.app.quit();
        return null;
    }
    browser_config.setupWindowState(win);
    if (browser_config.loaded_config !== null && browser_config.loaded_config.show_menubar === false) {
        win.setMenuBarVisibility(false);
    }
    win.once('closed', function () {
        win = null;
    });
    win.loadURL(index_html);
    if (process.env.NODE_ENV !== 'production' && is_run_from_npm_package_on_darwin) {
        win.webContents.openDevTools({ mode: 'detach' });
    }
    return win;
}
electron_1.app.once('window-all-closed', () => electron_1.app.quit());
electron_1.app.on('open-url', (e, u) => {
    e.preventDefault();
    electron_1.shell.openExternal(u);
});
electron_1.app.once('will-finish-launching', function () {
    electron_1.app.once('open-file', (e, p) => {
        process.argv.push(p);
        e.preventDefault();
    });
});
electron_1.app.once('ready', () => {
    if (process.platform === 'darwin' && is_run_from_npm_package_on_darwin) {
        electron_1.app.dock.setIcon(path_1.join(__dirname, '..', 'resources', 'icon', 'nyaovim-logo.png'));
    }
    Promise.all([
        ensure_nyaovimrc,
        prepare_browser_config,
    ]).then(() => {
        const w = startMainWindow();
        if (w !== null) {
            menu_1.default(w);
        }
    });
});
//# sourceMappingURL=main.js.map