"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const path_1 = require("path");
const fs_1 = require("fs");
class ComponentLoader {
    constructor() {
        this.initially_loaded = false;
        this.component_paths = [];
    }
    loadComponent(path) {
        const link = document.createElement('link');
        link.rel = 'import';
        link.href = path;
        document.head.appendChild(link);
        this.component_paths.push(path);
    }
    loadPluginDir(dir) {
        const nyaovim_plugin_dir = path_1.join(dir, 'nyaovim-plugin');
        try {
            for (const entry of fs_1.readdirSync(nyaovim_plugin_dir)) {
                if (entry.endsWith('.html')) {
                    this.loadComponent(path_1.join(nyaovim_plugin_dir, entry));
                }
            }
            this.nyaovim_plugin_paths.push(dir);
        }
        catch (err) {
        }
    }
    loadFromRTP(runtimepaths) {
        for (const rtp of runtimepaths) {
            this.loadPluginDir(rtp);
        }
    }
}
class RuntimeApi {
    constructor(definitions) {
        this.definitions = definitions;
        this.client = null;
    }
    subscribe(client) {
        client.on('notification', this.call.bind(this));
        for (const name in this.definitions) {
            client.subscribe(name).catch();
        }
        this.client = client;
    }
    unsubscribe() {
        if (this.client) {
            for (const name in this.definitions) {
                this.client.unsubscribe(name);
            }
        }
    }
    call(func_name, args) {
        const func = this.definitions[func_name];
        if (!func) {
            return null;
        }
        return func.apply(func, args);
    }
}
const component_loader = new ComponentLoader();
const ThisBrowserWindow = electron_1.remote.getCurrentWindow();
const runtime_api = new RuntimeApi({
    'nyaovim:load-path': (html_path) => {
        component_loader.loadComponent(html_path);
    },
    'nyaovim:load-plugin-dir': (dir_path) => {
        component_loader.loadPluginDir(dir_path);
    },
    'nyaovim:edit-start': (file_path) => {
        ThisBrowserWindow.setRepresentedFilename(file_path);
        electron_1.remote.app.addRecentDocument(file_path);
    },
    'nyaovim:require-script-file': (script_path) => {
        require(script_path);
    },
    'nyaovim:call-global-function': (func_name, args) => {
        const func = window[func_name];
        if (func) {
            func.apply(window, args);
        }
    },
    'nyaovim:open-devtools': (mode) => {
        const contents = electron_1.remote.getCurrentWebContents();
        contents.openDevTools({ mode });
    },
    'nyaovim:execute-javascript': (code) => {
        if (typeof code !== 'string') {
            console.error('nyaovim:execute-javascript: Not a string', code);
            return;
        }
        try {
            eval(code);
        }
        catch (e) {
            console.error('While executing javascript:', e, ' Code:', code);
        }
    },
    'nyaovim:browser-window': (method, args) => {
        try {
            ThisBrowserWindow[method].apply(ThisBrowserWindow, args);
        }
        catch (e) {
            console.error("Error while executing 'nyaovim:browser-window':", e, ' Method:', method, ' Args:', args);
        }
    },
});
function prepareIpc(client) {
    electron_1.ipcRenderer.on('nyaovim:exec-commands', (_, cmds) => {
        for (const c of cmds) {
            client.command(c);
        }
    });
    electron_1.ipcRenderer.on('nyaovim:copy', () => {
        client.eval('mode()').then((value) => {
            if (value.length === 0) {
                return undefined;
            }
            const ch = value[0];
            const code = value.charCodeAt(0);
            if (ch === 'v'
                || ch === 'V'
                || code === 22) {
                client.input('"+y');
            }
        });
    });
    electron_1.ipcRenderer.on('nyaovim:select-all', () => {
        client.eval('mode()').then((value) => {
            if (value.length === 0) {
                return undefined;
            }
            const command = value[0] === 'n' ? 'ggVG' : '<Esc>ggVG';
            client.input(command);
        });
    });
    electron_1.ipcRenderer.on('nyaovim:cut', () => {
        client.eval('mode()').then((value) => {
            if (value.length === 0) {
                return undefined;
            }
            const ch = value[0];
            const num = value.charCodeAt(0);
            if (ch === 'v'
                || ch === 'V'
                || num === 22) {
                client.input('"+x');
            }
        });
    });
    electron_1.ipcRenderer.on('nyaovim:paste', () => {
        client.eval('mode()').then((value) => {
            if (value.length === 0) {
                return undefined;
            }
            let command;
            const ch = value[0];
            const code = value.charCodeAt(0);
            if (ch === 'v') {
                command = '"_d"+P';
            }
            else if (ch === 'V') {
                command = '"_d"+p';
            }
            else if (code === 22 || ch === 'n') {
                command = '"+p';
            }
            else if (ch === 'i') {
                command = '"+gp';
            }
            else if (ch === 'c') {
                command = '<C-r>+';
            }
            if (command) {
                client.command(`normal! ${command}`);
            }
        });
    });
}
Polymer({
    is: 'nyaovim-app',
    properties: {
        argv: {
            type: Array,
            value() {
                let electron_argc = 1;
                if ('electron' === path_1.basename(electron_1.remote.process.argv[0]).toLowerCase()) {
                    electron_argc = 2;
                }
                const a = process.env.NYAOVIM_E2E_TEST_RUNNING ? [] : electron_1.remote.process.argv.slice(electron_argc);
                a.unshift('--cmd', `let\ g:nyaovim_version="${electron_1.remote.app.getVersion()}"`, '--cmd', `set\ rtp+=${path_1.join(__dirname, '..', 'runtime').replace(' ', '\ ')}`);
                a.unshift('-n');
                return a;
            },
        },
        editor: Object,
    },
    ready() {
        const element = document.getElementById('nyaovim-editor');
        const editor = element.editor;
        editor.on('error', (err) => alert(err.message));
        editor.on('quit', () => ThisBrowserWindow.close());
        this.editor = editor;
        editor.store.on('beep', () => electron_1.shell.beep());
        editor.store.on('title-changed', () => {
            document.title = editor.store.title;
        });
        editor.on('process-attached', () => {
            const client = editor.getClient();
            client.listRuntimePaths()
                .then((rtp) => {
                component_loader.loadFromRTP(rtp);
                component_loader.initially_loaded = true;
            });
            runtime_api.subscribe(client);
            element.addEventListener('drop', e => {
                e.preventDefault();
                const f = e.dataTransfer.files[0];
                if (f) {
                    client.command('edit! ' + f.path);
                }
            });
            electron_1.remote.app.on('open-file', (e, p) => {
                e.preventDefault();
                client.command('edit! ' + p);
            });
            prepareIpc(client);
        });
        element.addEventListener('dragover', e => e.preventDefault());
        window.addEventListener('keydown', e => {
            if (e.keyCode === 0x1b && !editor.store.focused) {
                editor.focus();
            }
        });
    },
});
//# sourceMappingURL=nyaovim-app.js.map